package Tcl::pTk::DragDrop::LocalSite;

our ($VERSION) = ('0.94');

use strict;

use base qw(Tcl::pTk::DropSite);


Tcl::pTk::DragDrop::Common::Type('Tcl::pTk::DropSite', 'Local');

##### Currently Empty subclass of DropSite used for local drag/drops ####

1;
__END__
